/* $Id: et6000.c,v 1.9 1998/11/24 18:17:13 marcus Exp $
******************************************************************************

   Tseng Labs ET6000 integrated PLL driver

   Copyright (C) 1997-1998 Jim Ursetto	[jim.ursetto@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************

	This code was derived from the following source of information:
	[1] Trio/Virge integrated PLL driver
	[2] Prerelease VGADOC5 by Finn Thoegersen
	Copyright 1990,91 by Thomas
	Roell, Dinkelscherben, Germany

******************************************************************************
*/

#include <kgi/maintainers.h>

#define DRIVER_NAME     "Tseng Labs ET6000 integrated PLL clock driver"
#define DRIVER_REV      "$Revision: 1.9 $"

#define DEBUG_LEVEL     0

#include <kgi/module.h>
#include <Tseng/et6000.h>
#include <Tseng/et6000_io.h>

#define CLK0_PLL 0x03

#include "pll.h"

/* xf86 recommends N2 [i.e. post-VCO or p] > 0 for stable clock */

static const struct pll_limits dclk_pll =
{
	14318180,                       /* Fref                         */
	{ 1, 1 },                       /* Factor                       */
	/* changed this! OK so far... */
	{ 100000000, 270000000 },       /* FVCO range                   */
	{ 1, 3 },                       /* Post-VCO divider range       */
	{ 3, 129 },                     /* Dividend range               */           
	{ 3, 33 }                       /* Divisor range                */
};

static struct kgi_clock clock =
{
    
	"Tseng Labs",              /* Manufacturer         */
	"ET6000 integrated DAC",    /* Model                */
	GM_ALL,                         /* Supported modes      */
	0,                              /* Special capabilities */
	{ 0x7FFF, 0x7FFF },             /* Maximum resolution   */
	NULL,				/* Private data hook	*/
	CLK_TYPE_PROG,                  /* Clock type           */
	{{
		0, 135000000,    /* Clock range          */
	}}
};

/* An SDAC command word is as follows:
 * **      [0:6]   s->mul - 2
 * **      [8:12]  s->div - 2
 * **      [13:14] s->p
 * */

#define INVALID_COMMAND ((uint) -1)

static kgi_uint pll_command(struct pll_state *s, struct kgi_mode *mode)
{
	ASSERT(s->mul - 2 < 128);
	ASSERT(s->div - 2 < 32);
	ASSERT(s->p < 4);
    
	return  (s->mul - 2) | ((s->div - 2) << 8) | (s->p << 13);
}

static void pll_set(struct kgi_display *dpy, kgi_uint cmd)
{
	kgiu8 hold;
	DEBUG1("pll_set M=%d N=%d\n", cmd&0xFF, (cmd>>8)&0xFF);
    
    /* Set DCLK PLL parameters. */
	hold = PCI_in8(dpy, PCI_CRTDACwi);
	PCI_out8(dpy, (hold & PCI67_RES) | CLK0_PLL, PCI_CRTDACwi);
	PCI_out8(dpy, cmd & 0xFF, PCI_CRTDAC);         /* Write M and N to    */
	PCI_out8(dpy, (cmd >> 8) & 0xFF, PCI_CRTDAC);  /* the clock register. */
	PCI_out8(dpy, hold, PCI_CRTDACwi);

	/* Set clock select lines */
	CLK_out8(dpy, CLK0_PLL, 0);
}

#include "pll.inc"                         
