 /*
 * GWT - General Windowing Toolkit
 *
 * Copyright (C) 1998 MenTaLguY - mentalg@geocities.com
 *                    Rodolphe Ortalo - ortalo@laas.fr
 *                    Tristan Wibberley - bloater@ps.cus.umist.ac.uk
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: gwt.h,v 1.11 2000/06/09 11:34:44 ortalo Exp $
 */

#ifndef __GGI_GWT_H
#define __GGI_GWT_H

#include <ggi/gii.h>
#include <ggi/ggi.h>
#include <ggi/xmi.h>

#ifdef __cplusplus
extern "C" {
#endif

  /*
   * Defines the opaque types for applications
   */
#ifndef _BUILDING_LIBGWT
  typedef void *gwt_window_t;
  typedef void *gwt_region_t;
#endif
  /*
   * Library initialisation and shutdown
   */
  int gwtInit(void);
  int gwtExit(void);
  int gwtAttach(ggi_visual_t vis);
  int gwtDetach(ggi_visual_t vis);
  /*
   * Window creation/destruction
   */
  int gwtCreateWindow(gwt_window_t wparent, gwt_window_t *win);
  int gwtDestroy(gwt_window_t win);
  /* Root Window creation: special case */
  int gwtCreateRootWindow(gii_input_t inp, ggi_visual_t vis, gwt_window_t *win);
  /*
   * Z-Order management and access functions
   */
  int gwtRaise(gwt_window_t win, gwt_window_t above);
  int gwtLower(gwt_window_t win, gwt_window_t below);
  int gwtGetTopChild(gwt_window_t win, gwt_window_t *top);
  int gwtGetBottomChild(gwt_window_t win, gwt_window_t *bottom);
  int gwtGetPrevChild(gwt_window_t win, gwt_window_t *prev);
  int gwtGetNextChild(gwt_window_t win, gwt_window_t *next);
  /*
   * Windows hierarchy manipulation
   * (NB: The Z-Order provides the access functions.)
   */
  int gwtGetParent(gwt_window_t win, gwt_window_t *parent);
  int gwtReparent(gwt_window_t win, gwt_window_t newparent);
  /*
   * Region manipulation functions
   */
  int gwtRegionCreate(gwt_region_t *region);
  void gwtRegionDestroy(gwt_region_t region);
  int gwtRegionIsEmpty(gwt_region_t region);
  int gwtPointInRegion(gwt_region_t region, int x, int y);
  int gwtInRegion(gwt_region_t region, gwt_region_t to_test, int *total);
  int gwtRegionCopy(gwt_region_t dest, gwt_region_t src);
  int gwtRegionSetRect(gwt_region_t r, int left, int top, int width, int height);
  int gwtRegionAddRect(gwt_region_t r, int left, int top, int width, int height);
  int gwtRegionUnion(gwt_region_t res_r1, gwt_region_t r2);
  int gwtRegionIntersection(gwt_region_t res_r1, gwt_region_t r2);
  int gwtRegionDifference(gwt_region_t res_r1, gwt_region_t r2);
  int gwtRegionRect(gwt_region_t region, int *rectno, ggi_coord *origin, ggi_coord *size);
  /* Access to the GGI visual of a GWT window (if needed) */
  int gwtGetVisual(gwt_window_t win, ggi_visual_t *vis);
  /*
   * Private data pointer access
   */
  int gwtGetPrivate(gwt_window_t win, void* *priv);
  int gwtSetPrivate(gwt_window_t win, void* priv);

#if 0
  /*
   * Windows ID system
   */
  /* TODO: To be implemented (id.c). */ /* if people like it (not sure)... */
  gwt_id_t gwtGetID(gwt_window_t win);
  gwt_window_t gwtGetFromID(gwt_id_t id);
#endif
  /*
   * Show/Hide functions
   */
  int gwtShow(gwt_window_t win);
  int gwtHide(gwt_window_t win);
  int gwtIsHidden(gwt_window_t win);	/* is it _explicility_ hidden? (w/ gwtHide) */
  int gwtIsVisible(gwt_window_t win);	/* true iff ((not (implicitly or explicitly) hidden) or new) */
  /*
   * Windows geometry manipulation
   */
  int gwtMove(gwt_window_t win, int x, int y);
  int gwtResize(gwt_window_t win, int w, int h);
  int gwtSetGeometry(gwt_window_t win, gwt_region_t region);
  int gwtGetGeometry(gwt_window_t win, gwt_region_t region);
  int gwtGetVisibleRegion(gwt_window_t win, gwt_region_t region);
  int gwtGetInvalidRegion(gwt_window_t win, gwt_region_t region);
  int gwtTranslateCoordinates(gwt_window_t source, int sx, int sy,
			      gwt_window_t destination, int *dx, int *dy);
  /*
   * Validation/Invalidation of some regions
   */
  int gwtInvalidate(gwt_window_t win, gwt_region_t region);
  int gwtValidate(gwt_window_t win, gwt_region_t region);
  int gwtValidateAll(gwt_window_t win);
  int gwtInvalidateAll(gwt_window_t win);
  /*
   * Event management system
   */
  /* Single window event queue access */
  int gwtEventPoll(gwt_window_t win, gii_event_mask mask, struct timeval *t);
  int gwtEventRead(gwt_window_t win, gii_event *ev, gii_event_mask mask);
  int gwtEventSend(gwt_window_t win, gii_event *ev);
  int gwtEventLocate(gwt_window_t parent, gii_event *ev, gwt_window_t *event_window);
  int gwtSetFocus(gwt_window_t parent, gwt_window_t new_f);
  int gwtGetFocus(gwt_window_t parent, gwt_window_t *focussed);
  int gwtGetPointerLocation(gwt_window_t win, int *x, int *y);
  int gwtSetPointerLocation(gwt_window_t win, int x, int y);
  /*
   * Graphic context (GC) management
   */
  int gwtGetGCForeground(gwt_window_t win, ggi_pixel* color);
  int gwtSetGCForeground(gwt_window_t win, ggi_pixel color);
  int gwtGetGCBackground(gwt_window_t win, ggi_pixel* color);
  int gwtSetGCBackground(gwt_window_t win, ggi_pixel color);
  int gwtGetClipping(gwt_window_t win, gwt_region_t clip);
  int gwtSetClipping(gwt_window_t win, gwt_region_t clip);
  /*
   * Drawing functions
   */
  /* THIS API IS EXPERIMENTAL ! */
  /* Expect functions names and arguments types to change -- ortalo */
  /* These are pass-through to LibGGI functions */
  int gwtFlush(gwt_window_t win);
  int gwtDrawPixel(gwt_window_t win, int x, int y);
  int gwtPutPixel(gwt_window_t win, int x, int y, ggi_pixel pixel);
  int gwtDrawBox(gwt_window_t win, int x, int y, int w, int h);
  int gwtDrawLine(gwt_window_t win,int x,int y,int xe,int ye);
  int gwtCrossBlit(gwt_window_t src, int sx, int sy, int sw, int sh,
		   gwt_window_t dst, int dx, int dy);
  int gwtPutc(gwt_window_t win,int x,int y,char c);
  int gwtPuts(gwt_window_t win,int x,int y,const char *str);
  /* These are pass-through to LibXMI functions */
  miPaintedSet * gwtmiNewPaintedSet(gwt_window_t win);
  void gwtmiDeletePaintedSet(gwt_window_t win, miPaintedSet *paintedSet);
  void gwtmiClearPaintedSet(gwt_window_t win, miPaintedSet *paintedSet);
  miGC * gwtmiNewGC(gwt_window_t win, int npixels, const miPixel *pixels,
		    int nblendstages, const miBlendStage *blendstages);
  void gwtmiDeleteGC(gwt_window_t win, miGC *pGC);
  miGC * gwtmiCopyGC(gwt_window_t win, const miGC *pGC);
  void gwtmiSetGCAttrib(gwt_window_t win, miGC *pGC, miGCAttribute attribute, int value);
  void gwtmiSetGCAttribs(gwt_window_t win, miGC *pGC, int nattributes,
			 const miGCAttribute *attributes, const int *values);
  void gwtmiSetGCDashes(gwt_window_t win, miGC *pGC, int ndashes, const unsigned int *dashes, int offset);
  void gwtmiSetGCMiterLimit(gwt_window_t win, miGC *pGC, double miter_limit);
  void gwtmiSetGCPixels(gwt_window_t win, miGC *pGC, int npixels, const miPixel *pixels); /* npixels >=2 */
  void gwtmiDrawPoints(gwt_window_t win, miPaintedSet *paintedSet, const miGC *pGC, miCoordMode mode,
		       int npts, const miPoint *pPts);
  void gwtmiDrawLines(gwt_window_t win, miPaintedSet *paintedSet, const miGC *pGC, miCoordMode mode,
		      int npts, const miPoint *pPts);
  void gwtmiFillPolygon(gwt_window_t win, miPaintedSet *paintedSet, const miGC *pGC,
			miPolygonShape shape, miCoordMode mode, int npts, const miPoint *pPts);
  void gwtmiDrawRectangles(gwt_window_t win, miPaintedSet *paintedSet, const miGC *pGC,
			   int nrects, const miRectangle *pRects);
  void gwtmiFillRectangles(gwt_window_t win, miPaintedSet *paintedSet, const miGC *pGC,
			   int nrects, const miRectangle *pRects);
  void gwtmiDrawArcs(gwt_window_t win, miPaintedSet *paintedSet, const miGC *pGC, int narcs, const miArc *parcs);
  void gwtmiFillArcs(gwt_window_t win, miPaintedSet *paintedSet, const miGC *pGC, int narcs, const miArc *parcs);

  miPixmap * gwtmiNewPixmap(uint w, uint h, ggi_graphtype type);
  void gwtmiDeletePixmap(gwt_window_t win, miPixmap *pPixmap);
  void gwtmiCopyPaintedSetToVisual(gwt_window_t win, const miGC *pGC,
				   const miPaintedSet *paintedSet, miPoint offset);

  /*
   * Debugging functions
   */
  int gwtDebugPrintWindows(gwt_window_t win);  
  /*
   * Temporary drawing functions to help debug
   */
  int gwtDrawFrame(gwt_window_t win);
  int gwtDrawVisibleRegion(gwt_window_t win);
  int gwtDrawInvalidRegion(gwt_window_t win);
  int gwtFillscreen(gwt_window_t win);

  //#define gwtDrawRect(x) gwtDrawVisibleRegion(x)

#ifdef __cplusplus
} /* of extern "C" */
#endif

#endif /* __GGI_GWT_H */
